// Demonstration program for StarClass(tm)

// (c) Copyright 1991,1992 Loesgen Software, Inc.
//     All Worldwide Rights Reserved

// Written by: Brian Loesgen

// Version 1.05   Released: Jan 15 1992


////
//                                                                           //
// Note: There are 2 ways to run this demonstration: automatic and manual.   //
//       No intervention (keystrokes) are needed when running in automatic   //
//       mode. There are 2 ways to invoke automatic mode: choosing the       //
//       "view entire demo" option and selecting automatic, or starting the  //
//       demo with a numeric parameter (the delay factor)                    //
//                                                                           //
//       For the self-run mode, the demo relies on FT_PUTKEY to stuff the    //
//       keyboard (it can handle function keys which the KEYBOARD command    //
//       cannot. FT_PUTKEY is part of the Nanforum Toolkit which is          //
//       available on Nanforum.                                              //
//                                                                           //
//       Because many people are likely to not have the toolkit, and few     //
//       will even want to run the demo in autoMode, I've put a STATIC       //
//       function at the end of this program. If you have the toolkit,       //
//       and want to run the demo in autoMode, comment out the replacement   //
//       function.                                                           //
//                                                                           //
////



#include "starclas.ch"
#include "scwindow.ch"
#include "inkey.ch"
#include "class(y).ch"

#define DEMO_COLOR "w+/b, n/w,,,G+/b"
#define PROMPT_COLOR "r+/b"

STATIC nDelaySecs
STATIC autoMode := .F.

FUNCTION main( nDelay )
  LOCAL cDosColor := SETCOLOR(), prompts := {}, choices := {}, oHots, x
  LOCAL nStart := seconds(), nWhich, lDone, lQuitRequest := .F.
  LOCAL oDelayWin, GetList := {}

  SETCANCEL(.F.)
  SETKEY( K_ALT_C, {|| ConfirmQuit() })

  cDosColor := SETCOLOR()
  IF nDelay <> NIL
     nDelay     := VAL( nDelay )
     nDelaySecs := nDelay
     autoMode   := .T.
  ELSE
     nDelaySecs := 0
  ENDIF

  DEFAULT cDosColor TO "w, n/w"
  SET CURSOR OFF
  SET SCOREBOARD OFF
  SET DELETED ON

#IFDEF DEMO
  csydemoins()
#ENDIF

  DeskTop()

  OpenDBFs()

  // define hot keys
  oHots := HotKey():new()
  oHots:addKey("BROWSE", K_F2, {|| Window():topmost():zoom() } )

  // Set up menues

  // a tilde ("~") denotes a hot key

  AADD(Choices,  {" ~About StarClass",        {||  CW1Info() } } )

  AADD(Choices,  {" ~Windows",                   {||  dmoIntro(),  ;
                                                      dmoTitle(),  ;
                                                      dmoColor(),  ;
                                                      dmoDepth(),  ;
                                                      dmoDrag(),   ;
                                                      dmoZoom(),   ;
                                                      dmoHide(),   ;
                                                      dmoSend(),   ;
                                                      dmoMarquee()}  } )



  AADD(Choices,  {" ~Read Window",               {||  dmoRead(),  ;
                                                      dmoRead2()} } )

  AADD(Choices,  {" ~TB Family",                 {||  dmoTBIntro(),;
                                                      dmoTBSeek(), ;
                                                      dmoTBDrag(), ;
                                                      dmoTBZoom(), ;
                                                      dmoTBPrnt(), ;
                                                      dmoTBPrnt2(),;
                                                      dmoMult() } } )

  AADD(Choices,  {" T~imer Class",               {||  dmoTimer1(), ;
                                                      dmoTimer2()} } )

  AADD(Choices,  {" ~Hot Keys",                  {||  dmoHotKey() } } )

  AADD(Choices,  {" ~Scroll Bars",               {||  dmoScrollBar() } } )



  AADD(Choices,  {" ~Ordering Info",             {||  OrderInfo() } } )

  AADD(Choices,  {" ~Quit",                      {||  lQuitRequest := .T.  } } )

  // build an array of prompts out of the choices
  AEVAL(choices, {|ele| AADD(prompts, ele[1])})

  REPEAT
     SETCOLOR( DEMO_COLOR )
     dmoStart()
     IF ! autoMode
       IF YNConf("Run the _ENTIRE_ demo?")
          AEVAL(choices, {|ele| EVAL(ele[2]) } )
          nDelaySecs := 0 ; autoMode := .F.
       ELSE
          nDelaySecs := 0
          REPEAT
             nWhich := AChoose( 5, 5, prompts, "w/b" )
             IF nWhich > 0
                EVAL(choices[nWhich,2])
             ENDIF
          UNTIL nWhich = 0 .OR. lQuitRequest
       ENDIF
       SETCOLOR( DEMO_COLOR )
       lDone := !YNConf("Do you want to run the demo again?")
     ELSE
       // automatic run forever mode...
       FOR x := 2 TO LEN(choices) - 2
           EVAL( choices[x, 2] )
       NEXT
       lDone := .f.
     ENDIF
  UNTIL lDone 

  dmoFinale()
  dmoGoodbye()

SET CURSOR ON
SETCOLOR(cDosColor)
CLEAR
QUIT
RETURN( NIL )



**
* Startup screen
**
FUNCTION dmoStart
// Startup screen
  LOCAL oWin1 := Window():new(8, 14, 18, 65, "w+/BG")
  LOCAL GetList := {}, nKey, nStartTime := SECONDS(), lDone := .F.
  CLEAR TYPEAHEAD
  @1,0 SAY "     << Welcome to a demo of StarClass(tm) >>       " COLOR "W+/B" IN oWin1
  @3,0 SAY "  (c) Copyright 1991, 1992 Loesgen Software, Inc.   " IN oWin1
  @5,0 SAY "           All Worldwide Rights Reserved            " IN oWin1

  oWin1:addMarquee(8,0,"Press any key to start the demo...",.2)
  oWin1:display()
  DO WHILE (nKey := INKEY()) == 0 .AND. ! lDone
     Window():displayMarquee()
     lDone := autoMode .AND. ( SECONDS() - nStartTime ) > nDelaySecs
  ENDDO
  oWin1:kill()
RETURN( NIL )

**
* StarClass Info
**
FUNCTION cw1info
  LOCAL cScreen := SAVESCREEN(0,0,MAXROW(),MAXCOL()), cString
  SETCOLOR( DEMO_COLOR )
  cString := MEMOREAD("descrip.txt")
  @0,0 CLEAR TO MAXROW(),MAXCOL()
  SETCOLOR( "rb+/b")
  @0,0 TO MAXROW(),MAXCOL()
  SETCOLOR( DEMO_COLOR )
  MEMOEDIT( cString,1,2,MAXROW()-1, MAXCOL()-1,.F.,, 80)
  RESTSCREEN(0,0,MAXROW(),MAXCOL(),cScreen)
RETURN( NIL )


**
* Windows
**
FUNCTION DmoIntro
  LOCAL oWin1 := Window():new(6, 10, 19, 69, "w+/BG")
  CLEAR TYPEAHEAD
  oWin1:title := " Introduction to the Window Class "  // so it has a title for the initial ::display()
  @ 1, 1 SAY "    This system is heavily Window oriented             " IN oWin1
  @ 3, 1 SAY "    All user input/activities happen in windows        " IN oWin1
  @ 5, 1 SAY "    The Windows Class has an extensive set of methods  " IN oWin1
  @ 7, 1 SAY "    Several classes are descendants of the Window      " IN oWin1
  @ 8, 1 SAY "     class, meaning they also inherit the methods       " IN oWin1
  @10, 1 SAY " We will now see a demonstration of some of the Window  " COLOR DEMO_COLOR IN oWin1
  @11, 1 SAY " methods in action.                                     " COLOR DEMO_COLOR IN oWin1
  @11,21 SAY ContMsg() COLOR PROMPT_COLOR IN oWin1
  oWin1:display()
  GetInput( nDelaySecs )
  oWin1:kill()
RETURN( NIL )

FUNCTION dmoTitle
  LOCAL oText := Window():new(6, 10, 17, 67, "w+/BG")
  LOCAL nIndex := 1, nKey, lDone := .F., nStartTime := SECONDS()
  LOCAL aTitles := { " Window 1 ", " Text Window ", " Client Record ", ;
                   "< Inventory File >", " Employees ", NIL}
  CLEAR TYPEAHEAD
  oText:title := " Window 1 "  // so it has a title for the initial ::display()
  @ 1, 1 SAY "Windows can have optional titles, which, if present,  " IN oText
  @ 2, 1 SAY "will be displayed in the top-left portion of the      " IN oText
  @ 3, 1 SAY "window frame.                                         " IN oText
  @ 5, 1 SAY "The title can be changed at any time by assigning a   " IN oText
  @ 6, 1 SAY "new string to the ::title instance variable, as we are" IN oText
  @ 7, 1 SAY "doing now.                                            " IN oText
  @ 9, 1 SAY PAD(ContMsg(),54) COLOR PROMPT_COLOR IN oText
  oText:display()
  DO WHILE (nKey := GetInput(.3)) == 0 .AND. ! lDone
     oText:title := aTitles[nIndex]
     nIndex := IIF(nIndex+1>LEN(aTitles), 1, nIndex+1 )
     Window():refresh()
     lDone := autoMode .AND. ( SECONDS() - nStartTime ) > nDelaySecs*2.5
  ENDDO
  oText:kill()
RETURN( NIL )

FUNCTION dmoColor
  LOCAL oWin1 := Window():new(6, 5, 12, 63)
  LOCAL oWin2 := Window():new(11, 7, 15, 61, DEMO_COLOR)
  LOCAL aBackColors := { "B", "G", "BG", "GR", "BR", "W", "N" }
  LOCAL aColors := { "W+/B", "G+/B", "R+/B",  "B+/B", "N/B", "GR/B", "W/B", }
  LOCAL nForeIndex := 1, nBackIndex := 1, nKey, cThisColor
  LOCAL nStartTime := SECONDS(), lDone := .F.
  LOCAL aForeColors := { "W+", "G+", "R+",  "B+", "N", "GR", "W" }
  CLEAR TYPEAHEAD
  @ 1, 1 SAY "Colors can be changed at any time simply by reassigning " IN oWin1
  @ 2, 1 SAY "the ::frameColor and ::paneColor instance variables.    " IN oWin1

  @ 0, 1 SAY "Notice that this window is NOT changing colors.   " IN oWin2
  @ 2, 1 SAY  PAD(ContMsg(),54) COLOR PROMPT_COLOR IN oWin2
  oWin1:title := " Color Demonstration "
  Window():displayAll()
  DO WHILE (nKey := GetInput()) == 0 .AND. !lDone
     oWin1:paneColor := oWin1:frameColor := ;
          (cThisColor := aForeColors[nForeIndex]+"/"+aBackColors[nBackIndex])
     // change the color of all text strings to match the ::paneColor
     AEVAL( oWin1:says, {|ele| ele:color := cThisColor} )
     IF ++nForeIndex > LEN(aForeColors)
       nForeIndex := 1
       nBackIndex := IIF(nBackIndex+1>LEN(aBackColors), 1, nBackIndex+1 )
     ENDIF
     Window():refresh()
     lDone := autoMode .AND. ( SECONDS() - nStartTime ) > nDelaySecs
  ENDDO
  Window():killAll()

  oWin1 := Window():new(6, 10, 12, 68)
  oWin1:title := " Selectable Text Colors "
  oWin1:frameColor := oWin1:paneColor := DEMO_COLOR
  @ 1, 1 SAY "Note that text in a window can be in a color other than " IN oWin1
  @ 2, 1 SAY "the ::frameColor and ::paneColor.                       " IN oWin1
  @ 4, 1 SAY  PAD(ContMsg(),54)  COLOR PROMPT_COLOR IN oWin1
  oWin1:display()

  nForeIndex := 1 ; nStartTime := SECONDS(); lDone := .F.
  DO WHILE (nKey := GetInput()) == 0 .AND. !lDone
     oWin1:winSayAgain( 1, 40, "color", aColors[nForeIndex] )
     oWin1:winSayAgain( 2,  5, "::frameColor", aColors[nForeIndex] )
     oWin1:winSayAgain( 2, 22, "::paneColor", aColors[nForeIndex] )
     nForeIndex := IIF(nForeIndex+1>LEN(aColors), 1, nForeIndex+1 )
     Window():refresh()
     lDone := autoMode .AND. ( SECONDS() - nStartTime ) > nDelaySecs
  ENDDO
  oWin1:kill()
RETURN( NIL )

FUNCTION dmoDepth
  LOCAL oWin1 := Window():new(5, 10, 10, 45, "w/gr")
  LOCAL oWin2 := Window():new(7, 20, 12, 55, "n/BG")
  LOCAL oWin3 := Window():new(9, 30, 13, 65, "w+/R")
  LOCAL oWin4 := Window():new(20, 2, 22, 77, DEMO_COLOR)
  LOCAL nStartTime := SECONDS(), lDone := .F., nKey
  CLEAR TYPEAHEAD
  oWin1:title := " Window 1 "
  @0,1 SAY "Just a window with some text..." in oWin1

  oWin2:title := " Window 2 "
  @0,1 SAY "Well HELLO World!" in oWin2
  @2,1 SAY "This is text in Window 2 " in oWin2
  @3,1 SAY "   ... and this is more text." in oWin2

  oWin3:title := " Window 3 "
  @1,1 SAY "More text in Window 3..." in oWin3

  @0,1  SAY "Demo of Depth movement (::next method)." IN oWin4
  @0,45 SAY ContMsg()  COLOR PROMPT_COLOR IN oWin4

  oWin1:display() ; delay(30)
  oWin2:display() ; delay()
  oWin3:display() ; delay()
  oWin4:display() ; delay()

  DO WHILE (nKey := GetInput()) == 0 .AND. !lDone
     Window():next()
     lDone := autoMode .AND. ( SECONDS() - nStartTime ) > nDelaySecs
  ENDDO
  Window():killAll()
RETURN( NIL )

FUNCTION dmoDrag
  LOCAL oWin1  := Window():new(5, 10, 10, 45, DEMO_COLOR)
  LOCAL oWin2  := Window():new(7, 20, 12, 55, "n/BG")
  LOCAL oWin3  := Window():new(9, 30, 13, 65, "w+/R")
  LOCAL oWin4  := Window():new(6, 10, 12, 69, DEMO_COLOR)
  LOCAL oWin5  := Window():new(20, 4, 22, 75, DEMO_COLOR)
  LOCAL oIdle, x
  CLEAR TYPEAHEAD
  oWin1:title  := " Window 1 " ; oWin2:title := " Window 2 " ; oWin3:title := " Window 3 "
  @ 1, 1 SAY "Windows can be dragged around. Cursor/tab/PgUp/PgDn keys " IN oWin4
  @ 2, 1 SAY "will move the window.                                    " IN oWin4
  @ 4, 1 SAY "Press <esc> when done...                                 " COLOR PROMPT_COLOR IN oWin4
  @ 0, 1 SAY "Demonstration of Drag method."           COLOR DEMO_COLOR IN oWin5
  @ 0,31 SAY "Cursor keys to move, <esc> when done..." COLOR PROMPT_COLOR IN oWin5

  oWin4:explode := .F.
  owin1:display() ; owin2:display() ; owin3:display() ; owin5:display() ; owin4:display()

  IF autoMode
     // this uses inkey because we want an initial delay to allow people
     // to read the screen before the oIdle kicks in and moves it around
     INKEY( nDelaySecs )
     oIdle := Idle():new({|| .t.}, {|| StuffKeys()})
     StuffKeys(   {K_DOWN, K_DOWN, K_DOWN, K_DOWN, K_DOWN, K_DOWN, ;
                  K_DOWN, K_DOWN, K_DOWN, K_LEFT, K_LEFT, K_LEFT, ;
                  K_LEFT, K_LEFT, K_LEFT, K_LEFT, K_LEFT, K_LEFT, K_LEFT, ;
                  K_TAB,  K_TAB,  K_TAB,  K_TAB,                   ;
                  K_UP,   K_UP,   K_UP,   K_UP,   K_UP,   K_UP,   ;
                  K_UP,   K_UP,   K_UP,   K_UP,   K_UP,   K_UP,   ;
                  K_UP,   K_UP,   K_UP,   K_UP,   K_UP,   K_UP,   ;
                  K_LEFT, K_LEFT, K_LEFT, K_LEFT, K_LEFT, K_LEFT, ;
                  K_LEFT, K_LEFT, K_LEFT, K_LEFT, K_LEFT, K_LEFT, ;
                  K_LEFT, K_LEFT, K_LEFT, K_LEFT, K_LEFT, K_LEFT, K_LEFT, ;
                  K_HOME, K_PGDN, K_END,  K_PGUP, ;
                  K_HOME, K_PGDN, K_END,  K_PGUP, ;
                  K_HOME, K_PGDN, K_END,  K_PGUP, ;
                  K_HOME, K_PGDN, K_END,  K_PGUP, ;
                  K_HOME, K_PGDN, K_END,  K_PGUP, ;
                  K_HOME, K_PGDN, K_END,  K_PGUP, ;
                  K_HOME, K_PGDN, K_END,  K_PGUP, ;
                  K_ESC } )
  ENDIF

  oWin4:drag()
  Window():topmost():hide(8)
  Window():topmost():hide(7)
  Window():topmost():hide(4)
  Window():topmost():hide(2)
  Window():killAll()
  IF autoMode
     oIdle:kill()
  ENDIF
RETURN( NIL )


FUNCTION dmoZoom
  LOCAL oWin1 := Window():new( 5, 5,10,60,DEMO_COLOR), r := 4, cName
  LOCAL lDone := .F., nStartTime := SECONDS()
  CLEAR TYPEAHEAD
  @0,1 SAY "The ZOOM method toggles the window size between       " IN oWin1
  @1,1 SAY "the size it was created as and full-screen.           " IN oWin1
  IF ! autoMode
     @3,1 SAY "Press any key to ZOOM this window, <esc> when done...." COLOR PROMPT_COLOR IN oWin1
  ELSE
     @3,1 SAY "<< Sending Zoom message. Press Alt-C to Abort demo... >>" COLOR PROMPT_COLOR IN oWin1
  ENDIF
  oWin1:display()
  DO WHILE GetInput( nDelaySecs/3 ) <> K_ESC .AND. !lDone
     oWin1:zoom()
     lDone := autoMode .AND. ( SECONDS() - nStartTime ) > (nDelaySecs*2.5)
  ENDDO
  oWin1:kill()

  // zoom part 2
  lDone := .F. ; nStartTime := SECONDS()
  oWin1 := Window():new( 5, 5,7,60,DEMO_COLOR)
  cName := "John Smith    "

  @0,1 SAY "Enter Name: " GET cName COLOR "n/w" IN oWin1
  @0,30 SAY "Press any key to Zoom..." COLOR "bg/b" IN oWin1

  @ 3,1 SAY "You can also write text to coordinates outside of the current window" IN oWin1
  @ 4,1 SAY "boundaries. That text will then become visible when the window is zoomed" IN oWin1
  @ 5,1 SAY "or the window is resized to include it." IN oWin1
  @ 7,1 SAY "One possible application of this is to include help in your read windows," IN oWin1
  @ 8,1 SAY "but write the text outside of the normally displayed area. Pressing a ZOOM" IN oWin1
  @ 9,1 SAY "hotkey would bring the text into view without affecting the READ." IN oWin1
  @11,1 SAY "Another application would be to avoid screen clutter in a READ window. Just" IN oWin1
  @12,1 SAY "show something unique in the ZOOMed down window, say ITEM #. If the user" IN oWin1
  @13,1 SAY "wants more information they can ZOOM into it." IN oWin1
  IF autoMode
     @15,1 SAY "<< Toggling ZOOM state," + ContMsg() + " >>" COLOR PROMPT_COLOR IN oWin1
  ELSE
     @15,1 SAY "Press any key to toggle ZOOM state, <esc> to continue..." COLOR PROMPT_COLOR IN oWin1
  ENDIF
  oWin1:zoom()
  IF autoMode
     // delay to allow viewer to read the text
     INKEY( nDelaySecs * 4 )
  ENDIF
  nStartTime := SECONDS()
  DO WHILE GetInput( nDelaySecs/3 ) <> K_ESC .AND. !lDone
     oWin1:zoom()
     lDone := autoMode .AND. ( SECONDS() - nStartTime ) > nDelaySecs*2
  ENDDO
  oWin1:kill()
RETURN( NIL )



FUNCTION dmoHide
  LOCAL oText := Window():new(16, 2, 23, 56, DEMO_COLOR)
  LOCAL oWin2 := Window():new( 10, 30,12,60,"w+/r")
  LOCAL oWin3 := Window():new( 9, 12,15,49,"n/bg"), nMode
  LOCAL v1, v2, v3, v4, v5
  CLEAR TYPEAHEAD
  oWin2:title := " Window 2 " ; oWin3:title := " Window 3 "

  @0,1 SAY "This is text." IN oWin2

  v1 := "Dave      " ; v2 := "Smith         " ; v3 := "17 - 32nd Street       "
  v4 := "Smalltown, USA         " ; v5 := "12345-1234"
  @0,1 SAY "Name: "              IN oWin3
  @0,10 GET v1                   IN oWin3
  @0,21 GET v2                   IN oWin3
  @1,1 SAY "" IN oWin3
  @2,1 SAY "Address: " GET v3    IN oWin3
  @3,10  GET v4                   IN oWin3
  @4,10 GET v5                   IN oWin3


  Window():displayAll()
  @0,1 SAY "There are several different ways to hide a window." IN oText
  @2,1 SAY "The simplest is the HIDE method which simply makes" IN oText
  @3,1 SAY "it invisible (as well as any SAYs and GETs)."       IN oText
  @5,1 SAY ContMsg() COLOR PROMPT_COLOR IN oText
  Window():displayAll()
  GetInput( nDelaySecs )
  oWin3:hide() ; oWin2:hide() ; oText:clear()
  oText:resize(18,,22,71)

  @0,1  SAY "But who wants to just make a window disapear when we have effects?" IN oText
  FOR nMode = 1 TO 8
      oText:WinSayAgain(2,15," << Now doing implode mode  "+LTRIM(STR(nMode))+" >>","GR+/B")
      Window():displayAll()
      Window():topmost:implode(nMode)
      Window():topmost:implode(nMode)
  NEXT
  Window():killAll()
RETURN( NIL )


FUNCTION dmoSend()
  LOCAL box1, box2, cntr := 1, oText, lDone := .F., nStartTime := SECONDS()
  CLEAR TYPEAHEAD
  box1  := Window():new(5,5,15,38,"W+/BG")
  box2  := Window():new(5,42,15,75,"n/BG")
  oText := Window():new(17,5,22,75,DEMO_COLOR)

  oText:winSay(0,1, "The SEND method allows you to easily create status type windows")
  oText:winSay(1,1, "which can be used to inform your users of activities being done")
  oText:winSay(3,1, ContMsg() , PROMPT_COLOR)
  oText:display()
  DO WHILE INKEY(.7) == 0 .AND. !lDone
     DO CASE
        CASE cntr = 1
             box1:send(time()+" -- Indexing...")
             box2:send("Hey! No you're not!")
        CASE cntr = 2
             box1:send(time()+" -- Backing up...")
             box2:send(time()+" -- Indexing over here...")
        CASE cntr = 3
             box1:send(time()+" -- Reporting...")
        CASE cntr = 4
             box1:send(time()+" -- Taking a break...")
             box2:send(time()+" -- Erasing myself...")
        CASE cntr = 5
             box2:send(time()+" -- Recovering myself...")
     ENDCASE
     cntr++
     cntr := IIF(cntr>5,1,cntr)
     lDone := autoMode .AND. ( SECONDS() - nStartTime ) > nDelaySecs*10
  ENDDO
  Window():killAll()
RETURN( NIL )

FUNCTION dmoMarquee
  // Marquee demo
  LOCAL oWin1 := Window():new( 7, 7,11,42,DEMO_COLOR)
  LOCAL oWin2 := Window():new( 10, 30,12,60,"w+/r")
  LOCAL oWin3 := Window():new( 9, 16,19,46,"n/bg")
  LOCAL oWin4 := Window():new(15,20,17,54,"n/w")
  LOCAL factor, cString, cString2, cString3, nKey
  LOCAL nStartTime := SECONDS(), lDone := .F.
  CLEAR TYPEAHEAD

  oWin1:title := " Window 1 " ; oWin2:title := " Window 2 "
  oWin3:title := " Window 3 " ; oWin4:title := " Window 4 "
  cString := "This is the Marquee class...  Each window can contain Marquee objects, _at least_ one per line... They scroll through....  FOREVER...."
  cString2 := "Each MARQUEE object can has an ::INTERVAL exported instance variable, which allows you to set a minimum period between updates.... "
  cString3 := " Look at this effect! "
  oWin1:addMarquee( 0, 0, " " + cString )
  oWin1:addMarquee( 1, 0, "  " + cString )
  oWin1:addMarquee( 2, 0, "   " + cString )
  oWin2:addMarquee( ,,cString )
  factor := (oWin3:right - oWin3:left) / 2

  *oWin3:addMarquee( 0,      0, cString3, 0, factor+1 )
  *oWin3:addMarquee( 0, factor, cString3, 0, factor+1 )
  oWin3:addMarquee( 0,      0, cString3, 0, factor+1, "W+/BG")
  oWin3:addMarquee( 0, factor, cString3, 0, factor+1, "GR+/BG")

  oWin4:addMarquee( ,,cString )
  IF autoMode
     @3,0 SAY " Press Alt-C to abort demo...     " COLOR "*"+PROMPT_COLOR IN oWin3
  ELSE
     @3,0 SAY " Press any key to continue...     " COLOR "*"+PROMPT_COLOR IN oWin3
  ENDIF
  window():displayAll()
  DO WHILE (nKey := INKEY()) == 0 .AND. !lDone
     Window():displayMarquee()
     lDone := autoMode .AND. ( SECONDS() - nStartTime ) > nDelaySecs*2
  ENDDO
  Window():killAll()
RETURN( NIL )

**
*  Reads
**

FUNCTION dmoRead
  LOCAL oWin1 := Window():new(4, 10, 19, 66, "w+/R")
  LOCAL memvar1, memvar2, r, offset, nStartTime := SECONDS()
  LOCAL oIdle
  CLEAR TYPEAHEAD
  IF autoMode
     oIdle := Idle():new({|| oIdle:elapsed > .3}, {|| StuffKeys()})
     StuffKeys(   {K_END, K_BS, K_BS, K_BS, K_BS, K_BS, ASC("T"), ASC("h"), ASC("o"), ASC("u"), ASC("g"), ASC("h"), ASC("t"),;
                   ASC("W"), ASC("a"), ASC("s"), K_PGDN } )
  ENDIF

  oWin1:title := " This is a Read Window "
  @ 0,1 SAY "As well as text, which is added to a Window with     " in oWin1
  @ 2,1 SAY "    @ <row>, <col> SAY <text> IN <window>            " in oWin1
  @ 4,1 SAY "We can also have GETs:                               " in oWin1
  @ 6,1 SAY "    @ <row>, <col> GET <text> IN <window>            " in oWin1
  @ 8,1 SAY "... which also supports all the clauses that you're  " in oWin1
  @ 9,1 SAY "used to (VALID, COLOR, WHEN, etc...).                " in oWin1
  @11,1 SAY "We are now doing a READ inside this window....       " in oWin1
  memvar1 := "think  " ; memvar2 := "am     "
  @13, 6 SAY "I "             GET memvar1 IN oWin1
  @13,14 SAY ", therefore I " GET memvar2 IN oWin1

  @13,36 say "(in Living Color!)" IN oWin1
  r := 13 ; offset := 46
  oWin1:winSayAgain(r, offset+1, "C", "gr+/r")
  oWin1:winSayAgain(r, offset+2, "o", "bg+/r")
  oWin1:winSayAgain(r, offset+3, "l", "n+/r")
  oWin1:winSayAgain(r, offset+4, "o", "b+/r")
  oWin1:winSayAgain(r, offset+5, "r", "g+/r")
  oWin1:read()
  IF autoMode
     oIdle:kill()
     GetInput( nDelaySecs/2 )
  ENDIF
  oWin1:kill()
RETURN( NIL )

FUNCTION dmoRead2
// put a browse window on the screen, when <enter> is pressed, bring up
// a READ window                         
  LOCAL oRead   := CreateRead(1)
  LOCAL oBrowse := CreateTB(1)
  LOCAL oText   := Window():new(12, 5, 15, 45, DEMO_COLOR)
  LOCAL oIdle
  CLEAR TYPEAHEAD
  IF autoMode
     oIdle := Idle():new({||oIdle:elapsed > .2}, {|| StuffKeys()})
     StuffKeys( {K_DOWN, K_DOWN, K_ENTER, ASC("S"), ASC("o"), ASC("u"), ASC("t"), ASC("h"), K_ESC,;
                 K_DOWN, K_DOWN, K_ENTER, ASC("M"), ASC("a"), ASC("s"), ASC("c"), ASC("o"), ASC("t"),K_ESC,;
                 K_DOWN, K_DOWN, K_ENTER, K_DOWN, K_DOWN, K_PGDN, ;
                 K_DOWN, K_DOWN, K_ENTER, K_DOWN, K_DOWN, K_PGDN, K_ESC } )
     GO TOP
  ENDIF

  oRead:title   := " Edit Customer "    ;  oRead:explode := .F.
  oBrowse:title := " Browse Customer "
  oBrowse:allowDelete := oBrowse:allowAppend := oBrowse:autoseek := .F.

  @0,1 SAY "Press <enter> to bring up Read window"  IN oText
  @1,1 SAY "or press <esc> to continue...        "  IN oText

  // ta daa.....
  ON ENTER IN oBrowse oRead:read():hide()
  oText:display()
  oBrowse:browse()

  oText:resize(oText:top+1,,20,65)
  oText:clear()
  oText:winSay(0,1,"The association of the read window to the browse window")
  oText:winSay(1,1,"that you just saw was installed by the following UDC:")
  oText:winSay(3,2,"  ON ENTER IN oBrowse oRead:read():hide()  ","n/bg")
  oText:winSay(5,1,ContMsg() ,PROMPT_COLOR)
  oText:display()
  CLEAR TYPEAHEAD
  INKEY( nDelaySecs*2 )
  Window():killAll()
  IF autoMode
     oIdle:kill()
  ENDIF
RETURN( NIL )


**
* TBWin Family
**
FUNCTION dmoTBIntro
  LOCAL oTB, oIdle
  LOCAL oWin1 := Window():new(  13, 10, 18, 78, DEMO_COLOR)
  LOCAL oWin2 := Window():new(  12,  5, 22, 75, DEMO_COLOR)
  CLEAR TYPEAHEAD
  IF autoMode
     oIdle := Idle():new({||oIdle:elapsed > .1}, {|| StuffKeys()})
     StuffKeys( {K_DOWN, K_DOWN, K_DOWN, K_DOWN, K_DOWN, K_DOWN, K_DOWN, ;
                 K_DOWN, K_DOWN, K_DOWN, K_DOWN, K_DOWN, K_DOWN, K_ESC } )
  ENDIF
  GO TOP

  @0, 1 SAY "This is the TBWin class. It is descended from the Windows      " IN oWin1
  @1, 1 SAY "Class, and contains a TBrowse object in instance variable ::tb." IN oWin1
  @3, 1 SAY "Go ahead....                                                   " IN oWin1
  @3,14 SAY "cursor around!"  COLOR PROMPT_COLOR IN oWin1

  @0,1 SAY "This is the syntax (more or less) used to create and browse" IN oWin2
  @1,1 SAY "that TBWin object:" IN oWin2
  @8,3 SAY ContMsg()                               COLOR PROMPT_COLOR IN oWin2
  @3,3 SAY '  LOCAL oTB := TBWin():new(  2, 2, 12, 30, "w/b")          ' COLOR "w+/bg" IN oWin2
  @4,3 SAY '  oTB:title := " Customers "                               ' COLOR "w+/bg" IN oWin2
  @5,3 SAY '  oTB:addField("last_name")                                ' COLOR "w+/bg" IN oWin2
  @6,3 SAY '  oTB:browse()                                             ' COLOR "w+/bg" IN oWin2

  oTB := CreateTB(1)
  oTB:autoSeek := .F.
  oTB:tb:getColumn(1):editColor := "w+/g, w+/g"

//  These are examples of how to use the TBCol:preblock and TBCol:postblock
//  instance variables
//
//  oTB:tb:getColumn(1):postBlock ;
//       := { |oGet| IIF(EMPTY(oGet:buffer),;
//                      (TONE(1500,2), MessageBox("Field cannot be blank","w+/r",2),.F.),;
//                      .T.) }
//
//  oTB:tb:getColumn(1):preBlock ;
//       := { |oGet| IIF( TRIM(UPPER(last_name)) = "SMITH",;
//                      (TONE(1500,2), MessageBox("Can't edit records for a Smith!","w+/r",2),.F.),;
//                      .T.) }

  oWin1:display()
  oTB:display()

  SET COLOR TO
  oTB:browse()
  IF autoMode
     oIdle:kill()
  ENDIF
  oWin1:implode(7)
  oWin2:display()
  GetInput( nDelaySecs )
  Window():killAll()
RETURN( NIL )

FUNCTION dmoTBSeek
  LOCAL oWin2 := Window():new( 14, 10, 22, 70, DEMO_COLOR)
  LOCAL oTB, oIdle
  CLEAR TYPEAHEAD
  IF autoMode
     GOTO RECCOUNT()/2
     oIdle := Idle():new({||oIdle:elapsed > .5}, {|| StuffKeys()})
     StuffKeys( {ASC("M"), ASC("O"), ASC("T"), K_ESC, K_ESC })
  ENDIF
  GO TOP

  @0,1  SAY "Each TBWin object has an incremental search feature for   " IN oWin2
  @1,1  SAY "doing indexed seeks. All you need to do is start typing,  "  IN oWin2
  @2,1  SAY "the TBWin object will attempt to find what you are typing."  IN oWin2
  @4,1  SAY "Pressing <backspace> deletes a character from the seek    "  IN oWin2
  @5,1  SAY "string and takes you back to the last match, a tone means " IN oWin2
  @6,1  SAY "that no match was found.                                  " IN oWin2
  @6,26 SAY "Press <esc> when done..." COLOR PROMPT_COLOR IN oWin2

  oTB := CreateTB(1)
  SET COLOR TO
  oWin2:display()
  oTB:browse()
  IF autoMode
     oIdle:kill()
  ENDIF

  oWin2:clear()
  oWin2:resize( 15,10,21,72)
  @0,1 SAY "All of that was handled by the browse method. The box " IN oWin2
  @1,1 SAY "containing the seek string was a window object contained in" IN oWin2
  @2,1 SAY "the TBWin object.                                          " IN oWin2
  @4,1 SAY ContMsg() COLOR PROMPT_COLOR IN oWin2
  oWin2:display()
  GetInput( nDelaySecs )

  Window():killAll()
RETURN( NIL )

FUNCTION dmoTBDrag
  LOCAL newTop, newLeft, x, oTB := CreateTB(1), cntr := 1
  CLEAR TYPEAHEAD
  oTb:display()
  FOR cntr = oTB:bottom TO MAXROW()
     oTB:move(oTB:top+1, 1)
  NEXT
  FOR cntr = oTB:right TO MAXCOL() STEP 2
     oTB:move(oTB:top-1, oTB:left+2)
  NEXT
  FOR x = 1 TO 15
     oTB:move( Random(MAXROW()-oTB:bottom+oTB:top)+1,;
               Random(MAXCOL()-oTB:right+oTB:left)+1)
  NEXT
  oTB:implode(7)
  Window():killAll()
RETURN( NIL )

FUNCTION dmoTBZoom
  LOCAL oTB := CreateTB(1), oIdle, oIdle2
  LOCAL oText := Window():new(13,7,22,71,DEMO_COLOR)
  CLEAR TYPEAHEAD
  IF autoMode
     oIdle := Idle():new({||oIdle:elapsed > .3}, {|| oTB:zoom() })
     oIdle2:= Idle():new({||oIdle2:elapsed > .2}, {|| StuffKeys()})
     StuffKeys( {K_DOWN, K_DOWN, K_DOWN, K_F2, K_DOWN, K_RIGHT, K_F2, K_DOWN, K_LEFT, ;
                 K_DOWN, K_DOWN, K_DOWN, K_F2, K_DOWN, K_RIGHT, K_DOWN, K_F2, ;
                 K_DOWN, K_RIGHT, K_DOWN, K_F2, K_DOWN, K_LEFT, K_DOWN, K_F2, K_ESC } )
  ENDIF
  GO TOP

  @0,1 SAY "Remember the ZOOM method we looked at in the Window class?   " IN oText
  @2,1 SAY "Well.... the TBGen family is descended from the Window class," IN oText
  @3,1 SAY "so we can invoke that method (with some additions) from here." IN oText
  @5,1 SAY "Note what the horizontal scroll indicator does when you Zoom." IN oText
  @7,1 SAY "Press F2 to ZOOM, <esc> when done..." COLOR PROMPT_COLOR IN oText

  oText:display()

  oTB:addField("city") ; oTB:addField("telephone","@R 999-999-9999")
  oTB:browse()
  IF autoMode
     oIdle:kill() ; oIdle2:kill()
  ENDIF
  Window():killAll()
RETURN( NIL )


FUNCTION dmoTBPrnt
  LOCAL oText := Window():new(14,4,23,75,DEMO_COLOR)
  LOCAL oParent := CreateTB(1), oChild := CreateTB(2)
  LOCAL oIdle
  CLEAR TYPEAHEAD
  IF autoMode
     oIdle := Idle():new({||oIdle:elapsed > .5}, {|| StuffKeys()})
     StuffKeys( {K_DOWN, K_DOWN, K_DOWN, K_TAB,  K_DOWN, K_DOWN, K_DOWN, ;
                 K_TAB,  K_DOWN, K_DOWN, K_DOWN, K_ESC,  K_ESC })
  ENDIF
  people->(dbGotop())

  @0,1 SAY "You are now looking at 2 TBWin objects, CUSTOMERS and ORDERS, "   IN oText
  @1,1 SAY "which have been related by PARENT command:"                       IN oText
  @3,1 SAY "    PARENT oParent CHILD oChild KEY upper(people->last_name)  "   IN oText
  @5,1 SAY "As you would expect, any movement through the PARENT window will" IN oText
  @6,1 SAY "cause the CHILD window to be updated, with a lookup done on the"  IN oText
  @7,1 SAY "KEY clause. Go ahead, try it, "                                   IN oText
  @7,31 SAY "cursor around. Press <esc> when done..." COLOR PROMPT_COLOR IN oText

  PARENT oParent CHILD oChild KEY upper(people->last_name) KEYFIELD last_name
  oText:display()
  oParent:browse()
  Window():killAll()
  IF autoMode
     oIdle:kill()
  ENDIF
RETURN( NIL )

FUNCTION dmoTBPrnt2
  LOCAL oText := Window():new(13,2,19,77,DEMO_COLOR), oIdle
  LOCAL oParent := CreateTB(1), oChild1 := CreateTB(2), oChild2 := CreateTB(3)
  CLEAR TYPEAHEAD

  @0,1  SAY "Each PARENT can have multiple CHILD TBWin objects, and a CHILD can"   IN oText
  @1,1  SAY "in turn be a PARENT. Displaying a PARENT will automatically display"  IN oText
  @2,1  SAY "the children. You are about to see one parent (Customer) with 2 children." IN oText
  @4,1  SAY ContMsg()  COLOR PROMPT_COLOR IN oText

  PARENT oParent CHILD oChild1 KEY upper(people->last_name) KEYFIELD last_name
  PARENT oParent CHILD oChild2 KEY upper(people->last_name) KEYFIELD last_name
  oText:display()
  GetInput( nDelaySecs * 2 )
  oText:implode(7)

  oText:clear()
  oText:resize(13,0,20,30)
  @0,1 SAY "Press TAB to move between" in oText
  @1,1 SAY "TBWin objects (if they   " in oText
  @2,1 SAY "           have any data)" in oText
  @4,1 SAY "Press <esc> when done..." in oText
  oText:display()
  IF autoMode
     oIdle := Idle():new({||oIdle:elapsed > .5}, {|| StuffKeys()})
     StuffKeys( {K_DOWN, K_DOWN, K_DOWN, K_TAB,  K_DOWN, K_DOWN, K_DOWN, ;
                 K_TAB,  K_DOWN, K_DOWN, K_DOWN, K_ESC,  K_ESC })
  ENDIF
  people->(dbGotop())
  oParent:browse()
  Window():killAll()
  IF autoMode
     oIdle:kill()
  ENDIF
RETURN( NIL )


FUNCTION dmoMult
// the piece de resistance... putting it all together
  LOCAL oReadPeople  := CreateRead(1), oReadOrders := CreateRead(2)
  LOCAL oBrPeople    := CreateTB(1), oBrOrders := CreateTB(2)
  LOCAL oText        := Window():new(5, 10, 17, 70, DEMO_COLOR)
  LOCAL oIdle
  CLEAR TYPEAHEAD

  oBrPeople:addField("city")

  @ 0,1 SAY "And now we come to the piece-de-resistance. It's time to" IN oText
  @ 1,1 SAY "put some of these objects together and see what we come"  IN oText
  @ 2,1 SAY "up with. Make sure you look at the source code for what"  IN oText
  @ 3,1 SAY "you're about to see.                                   "  IN oText

  @ 5,1 SAY "You'll be looking at 2 browse windows (TBWin objects), "  IN oText
  @ 6,1 SAY 'which have been related using the "PARENT" command.    '  IN oText

  @ 8,1 SAY "Pressing <enter> in either one will bring up a READ    "  IN oText
  @ 9,1 SAY "window, which was associated with the browse window    "  IN oText
  @10,1 SAY 'by the "ON ENTER" command.                             '  IN oText

  IF autoMode
     // in order for this demo to be meaningfull in autoMode, we need these records...
     SELECT people
     SEEK "JONES"
     IF ! FOUND()
        APPEND BLANK
        REPLACE last_name WITH "JONES",first_name WITH "Jim", city WITH "Montreal",;
                address WITH "45 TB-Window Avenue",telephone WITH "123-1212"
     ENDIF
     SELECT orders
     SEEK "JONES"
     IF ! FOUND()
        APPEND BLANK
        REPLACE last_name WITH "JONES", item WITH "BOLT", quantity WITH 3, price WITH 17
        APPEND BLANK
        REPLACE last_name WITH "JONES", item WITH "SCREW", quantity WITH 4, price WITH 22
        APPEND BLANK
        REPLACE last_name WITH "JONES", item WITH "LUGNUT", quantity WITH 2, price WITH 8
     ENDIF
  ENDIF


  oText:display()
  GetInput( nDelaySecs )
  IF autoMode
     oIdle       := Idle():new({||oIdle:elapsed > .5}, {|| StuffKeys()})
     StuffKeys( {K_ENTER, K_ENTER, K_DOWN, K_DOWN, K_DOWN, K_ESC, K_TAB, K_DOWN, K_DOWN, K_ENTER,;
                 K_DOWN, K_ESC, K_DOWN, K_TAB, K_DOWN, K_DOWN, K_ENTER, K_ESC,  K_ESC })
     people->(dbGotop())
  ENDIF
  oText:implode(7)


  oText:clear()
  oText:resize(17, 5, 21, 44)
  @0,1 SAY "<enter> to bring up Read window     "  IN oText
  @1,1 SAY "<tab> to move between browse windows"  IN oText
  @2,1 SAY "<esc> when done                     "  IN oText

  // ta daa..... Amazing what these few lines do, huh?
  PARENT oBrPeople CHILD oBrOrders KEY UPPER(people->last_name) KEYFIELD last_name
  ON ENTER IN oBrOrders oReadOrders:read():hide()
  ON ENTER IN oBrPeople oReadPeople:read():hide()

  ON ADD IN oBrOrders oReadOrders:read():hide()
  ON ADD IN oBrPeople oReadPeople:read():hide()

  // Note: The following PACKs are to ensure there aren't any deleted
  //       records in the DBF, which would cause inaccurate scroll bar display.
  SELECT people
  PACK
  SELECT orders
  PACK

  oText:display()
  oBrPeople:browse()

  Window():killAll()
  IF autoMode
     oIdle:kill()
  ENDIF
RETURN( NIL )




**
* Timer
**
FUNCTION dmoTimer1
  LOCAL oTimer := timer():new(5,5,,"n/bg")
  LOCAL oText  := window():new(18,10,23,70, DEMO_COLOR)
  CLEAR TYPEAHEAD

  oTimer:flipsize()
  oTimer:displaySeconds(.T.)
  @0,1 SAY "This is a TIMER object, which contains the current time." IN oText
  @1,1 SAY "We could have created it with a start time which would  " IN oText
  @2,1 SAY "have made it an elapsed time counter.                   " IN oText
  @3,1 SAY ContMsg() COLOR PROMPT_COLOR IN oText
  Window():displayAll()
  GetInput( nDelaySecs )

  oTimer:displaySeconds(.F.)
  oText:clear()
  @0,1 SAY "The TIMER object is now no longer displaying seconds. " IN oText
  @3,1 SAY ContMsg() COLOR PROMPT_COLOR IN oText
  Window():displayAll()
  GetInput( nDelaySecs )

  oTimer:flipsize()
  oText:clear()
  @0,1 SAY "WHOAAAA...... Will ya look at that!!!!                 " IN oText
  @3,1 SAY ContMsg() COLOR PROMPT_COLOR IN oText
  Window():displayAll()
  GetInput( nDelaySecs )

  oTimer:displaySeconds(.T.)
  oText:clear()
  @0,1 SAY "Now we're showing seconds again in large mode.         " IN oText
  @3,1 SAY ContMsg() COLOR PROMPT_COLOR IN oText
  Window():displayAll()
  GetInput( nDelaySecs )
  *oTimer:kill()
  *oText:kill()
  Timer():killAll() ; Window():killAll()
  
RETURN( NIL )


FUNCTION dmoTimer2
  LOCAL oWin1 := Window():new(16,5,22,75, DEMO_COLOR), oWin2 := timer():new(2,5,,"w+/bg")
  LOCAL oWin3 := timer():new( 6, 2, SECONDS()-300, "n/g+" )
  LOCAL oWin4 := timer():new( 7, 30, SECONDS()-200, "n/gr" )
  LOCAL oWin5 := timer():new( 7, 60, SECONDS()-100, "w+/r" )
  CLEAR TYPEAHEAD
  oWin2:title := " Timer Window 2 " ; oWin4:flipsize() ; oWin5:flipsize()
  @0,1 SAY "The timer class is descended from Windows, so we can apply most of "  IN oWin1
  @1,1 SAY "the Window methods (some get redefined). This whole screen, by the "  IN oWin1
  @2,1 SAY "way, took about 12 lines of code (INCLUDING 4 lines for this text) "  IN oWin1
  @4,1 SAY ContMsg() COLOR PROMPT_COLOR IN oWin1
  oWin2:display() ; oWin3:display() ; oWin4:display() ; oWin5:display() ;oWin1:display()
  GetInput( nDelaySecs )

  Window():topmost():implode(8) ; Window():refresh() // The refresh method callis to update the timers
  Window():topmost():implode(1) ; Window():refresh() // is to update the timers
  Window():topmost():implode(3) ; Window():refresh()
  Window():topmost():implode(7) ; Window():refresh()
  Window():topmost():implode(7)
  Window():killAll()
RETURN( NIL )

**
*  Hot Keys
**

FUNCTION dmoHotKey
  LOCAL oText := Window():new( 2, 5,18,75,DEMO_COLOR)
  LOCAL oIdle, keys := HotKey():new()

  CLEAR TYPEAHEAD
  keys:addKey("GROUP1", K_F4, {|| MessageBox("You pressed F4 (defined in Group 1). Press <esc> to continue...")})
  keys:activate("GROUP1")

  @ 0,1 SAY "The HotKey class allows for easy handling of hot key assignments." IN oText
  @ 2,1 SAY 'Assignments are broken down into "groups" of hotkeys. Examples of' IN oText
  @ 3,1 SAY "groups would be AR hot keys or AP hot keys. The AR group could   " IN oText
  @ 4,1 SAY "contain an assignement to F5 which will show clients, whereas in " IN oText
  @ 5,1 SAY "the AP group F5 could be redefined to show suppliers.              " IN oText
  @ 7,1 SAY "Once the group is initialized, it has application-wide scope.    " IN oText
  @ 9,1 SAY "Groups can be deactivated, which will set their member keys to   " IN oText
  @10,1 SAY "NIL. If you do not deactivate a group before activating another, " IN oText
  @11,1 SAY "the hot key assignments from the first group will be retained    " IN oText
  @12,1 SAY "unless they are specifically reassigned in the second group.     " IN oText
  @14,1 SAY ContMsg() COLOR PROMPT_COLOR IN oText

  oText:display()
  GetInput( nDelaySecs )
  oText:resize(2,2,10,78)
  oText:clear()
  oText:winSay(0,1, "The following code has just run:")
  oText:winSay(2,1, "LOCAL keys := HotKey():new()                                             ", "w+/bg")
  oText:winSay(3,1, 'keys:addKey("GROUP1", K_F4, {|| MessageBox("You pressed F4, (Group 1)")})', "w+/bg")
  oText:winSay(4,1, 'keys:activate("GROUP1")                                                  ', "w+/bg")
  oText:winSay(6,1, "Go ahead...  Press F4...", PROMPT_COLOR)
  oText:display()
  IF autoMode
     GetInput( nDelaySecs )
     oIdle := Idle():new({||oIdle:elapsed > nDelaySecs }, {|| StuffKeys()})
     StuffKeys( {K_F4, K_ESC} )
  ENDIF
  WHILE GetInput() <> K_F4 ; END


  keys:addKey("GROUP2", K_F3, {|| MessageBox("You pressed F3 (defined in Group 2). Press <esc> to continue...")})
  keys:activate("GROUP2")
  oText:clear()
  oText:resize(2,2,9,78)
  oText:winSay(0,1, "Now lets add another group. The following code has just run:")
  oText:winSay(2,1, 'keys:addKey("GROUP2", K_F3, {|| MessageBox("You pressed F3, (Group 2)")})', "w+/bg")
  oText:winSay(3,1, 'keys:activate("GROUP2")                                                  ', "w+/bg")
  oText:winSay(5,1, "Go ahead...  Press F3...", PROMPT_COLOR )
  oText:display()
  IF autoMode
     oIdle:active := .F. ; GetInput( nDelaySecs ) ; oIdle:active := .T.
     StuffKeys( {K_F3, K_ESC} )
  ENDIF
  WHILE GetInput() <> K_F3 ; END

  oText:clear()
  oText:winSay(0,1, "Both groups of hotkeys are now active because we didn't deactivate")
  oText:winSay(1,1, '"GROUP1" first. As each group only contained 1 hotkey (they could have')
  oText:winSay(2,1, 'contained as many as we wanted) and the key in GROUP2 did not overwrite')
  oText:winSay(3,1, 'the key assignment in GROUP1, GROUP1 remained active.')
  oText:winSay(5,1, ContMsg() , PROMPT_COLOR)
  oText:display()
  IF autoMode
     oIdle:active := .F. ; GetInput( nDelaySecs ) ; oIdle:active := .T.
     StuffKeys( {K_F3, K_F4, K_F3, K_F4, K_ESC} )
  ENDIF
  GetInput( nDelaySecs )
  oText:kill()
  IF autoMode
     oIdle:kill()
  ENDIF
RETURN( NIL )




**
*  Scroll Bars
**

FUNCTION dmoScrollBar
  LOCAL oTB := CreateTB(2), oHots
  LOCAL oText := Window():new(11,5,23,75, DEMO_COLOR)
  LOCAL oIdle, oIdle2
  CLEAR TYPEAHEAD
  IF autoMode
     oIdle       := Idle():new({||oIdle:elapsed > .5}, {|| ScrollSwap( oTB ), oTB:displayBars()})
     oIdle2      := Idle():new({||oIdle2:elapsed > 1}, {|| StuffKeys()})
     StuffKeys( { K_DOWN, K_DOWN, K_DOWN, K_RIGHT, K_RIGHT, K_DOWN, K_DOWN, K_ESC})
  ENDIF

  oTB:addfield("ship_date")

  // set up hot keys for this part of the demo
  oHots := HotKey():new()
  oHots:addKey( "SCROLL", K_F2, {|| oTB:zoom()} )
  oHots:addKey( "SCROLL", K_F4, {|| scrollSwap( oTB ), oTB:displayBars() } )
  oHots:addKey( "SCROLL", K_F6, {|| oTB:drag() } )
  oHots:activate("SCROLL")

  // Turn off index so vertical scroll bar makes sense
  SET ORDER TO 0
  GO TOP

  oTB:title := " This is a TBWin Object "
  oTB:move(2,45)
  oTB:showHBar := oTB:showVBar := .T.
  oText:winSay(0,1,"A TBWin object can contain vertical and horizontal scroll bars.")
  oText:winSay(2,1,"The horizontal scroll bar will be added to the window if all the")
  oText:winSay(3,1,"fields (columns) you've added will not fit into the window.")
  oText:winSay(5,1,"The vertical scroll bar, by default, shows the pointer position")
  oText:winSay(6,1,"in a DBF based on the RECNO().")
  oText:winSay(8,1,"Both scroll bars are instances of the BAR class.")
  oText:winSay(10,1,"Cursor around, or press <esc> when done...", PROMPT_COLOR)
  oText:display()
  oTB:browse()
  oTB:implode(3) ; oText:hide(1) ; oText:clear()


  IF autoMode
     StuffKeys( { K_DOWN, K_RIGHT, K_DOWN, K_RIGHT, K_RIGHT, K_DOWN, K_LEFT, K_ESC})
  ENDIF

  ScrollSwap( oTB )
  oText:resize(oText:top, oText:left, oText:bottom-1, oText:right)
  oText:winSay(0,1,"You have complete control over the visual appearance of the")
  oText:winSay(1,1,"scroll bars. You can define:")
  oText:winSay(3,1,"                       Bar character","BG+/B")
  oText:winSay(4,1,"                       Bar color","BG+/B")
  oText:winSay(5,1,"                       Pointer character","BG+/B")
  oText:winSay(6,1,"                       Pointer color","BG+/B")
  oText:winSay(8,1,"Cursor around, <F2> to zoom, <F4> to swap scroll bar characters", PROMPT_COLOR)
  oText:winSay(9,1,"and colors, <F6> to drag, or press <esc> when done...", PROMPT_COLOR)
  oText:display()
  oTB:browse()
  oTB:implode(2) ; oText:implode(4) ; oText:clear()

  Window():killAll()
  oHots:deactivate("SCROLL")
  IF autoMode
     oIdle:kill() ; oIdle2:kill()
  ENDIF
  SET ORDER TO 1
RETURN( NIL )

STATIC FUNCTION ScrollSwap( oTB, nChoice )
  STATIC nSet := 0
  nSet := IF(nChoice <> NIL, nChoice, nSet)
  DO CASE
     CASE nSet = 0
        oTB:hbar:fillcharacter := ""     ; oTB:hbar:pointerCharacter := ""
        oTB:hbar:fillColor     := "bg/b"  ; oTB:hbar:pointerColor     := "g+/r"

        oTB:vbar:fillcharacter := ""     ; oTB:vbar:pointerCharacter := ""
        oTB:vbar:fillColor     := "bg/b"  ; oTB:vbar:pointerColor     := "g+/r"

     CASE nSet = 1
        oTB:hBar:fillCharacter  := ""            ; oTB:hBar:pointerCharacter := ""
        oTB:hBar:fillColor      := "n/bg+, n/bg+" ; oTB:hBar:pointerColor := "r+/n+"

        oTB:vBar:fillCharacter  := ""         ; oTB:vBar:pointerCharacter := ""
        oTB:vbar:fillColor      := "n/bg+"     ; oTB:vbar:pointerColor     := "r+/n+"

     CASE nSet = 2
        oTB:hbar:fillcharacter := ""     ; oTB:hbar:pointerCharacter := ""
        oTB:hbar:fillColor     := "bg/b+"  ; oTB:hbar:pointerColor     := "g+/b+"

        oTB:vbar:fillcharacter := ""     ; oTB:vbar:pointerCharacter := ""
        oTB:vbar:fillColor     := "bg/b+"  ; oTB:vbar:pointerColor     := "g+/b+"
  ENDCASE
  nSet++ ; nSet := IIF(nSet>2,0,nSet)
RETURN( NIL )



**
* Order Info
**
FUNCTION OrderInfo
  LOCAL cScreen := SAVESCREEN(0,0,MAXROW(),MAXCOL())
  SETCOLOR( DEMO_COLOR )
  @0,0 CLEAR TO MAXROW(),MAXCOL()
  @0,0 TO MAXROW(),MAXCOL()
  MEMOEDIT(MEMOREAD("order.frm"),1,1,MAXROW()-1, MAXCOL()-1,.F.,,80)
  IF YNCONF("Print order form on PRN?")
     !COPY order.frm prn
     EJECT
  ENDIF
  RESTSCREEN(0,0,MAXROW(),MAXCOL(),cScreen)
RETURN( NIL )



**
* Closing Screens
**
FUNCTION dmoFinale
  LOCAL tim1 := Timer():new(5,5,,"W+/GR"), tim2 := Timer():new(10,7,9600,"W/G")
  LOCAL oText, x
  LOCAL oWin1 := Window():new(12,12,16,42,"w+/br")
  LOCAL oWin2 := Window():new(15,30,19,62,"n/bg")
  LOCAL oWin3 := Window():new(8,25,12,60,"b+/r")
  LOCAL oWin4 := Window():new(12,25,16,64,"gr+/b")
  CLEAR TYPEAHEAD

  @1,1 SAY "He's pining for the fjords!"  IN oWin1
  @1,1 SAY "No parots, how about a slug?" IN oWin2
  @1,1 SAY "That's not a Norwegian Blue!" IN oWin3
  @1,1 SAY "He's not dead, he's merely resting." IN oWin4
  Window():displayAll()
  FOR x = 1 TO 30
     Window():next()
     IF INKEY() <> 0 ; x := 31 ; ENDIF
  NEXT
  Window():killAll()
RETURN( NIL )


FUNCTION dmoGoodbye
  LOCAL oText := Window():new(6,4,21,76,DEMO_COLOR), GetList := {}, nKey
  CLEA TYPEAHEAD
  SET COLOR TO W/BG
  CLEAR
  oText:winSay( 0,0,"           This concludes the StarClass(tm) demonstration.             ", "G+/BG")
  oText:winSay( 2,1,"This is very much a user driven product. We're open to any comments")
  oText:winSay( 3,1,"or suggestions you may have. We can be reached for feedback and ")
  oText:winSay( 4,1,"orders at:")
  oText:winSay( 6,1,"                  Loesgen Software, Inc.")
  oText:winSay( 7,1,"                  1712 Avenue Rd, Box 54605")
  oText:winSay( 8,1,"                  Toronto, ON")
  oText:winSay( 9,1,"                  Canada M5M 4N5")
  oText:winSay(11,1,"  CIS: 74326,1174    Fax: 416-784-3618    Voice: 416-784-3618  ")
  oText:addMarquee(,,"Only $149.00 US (plus shipping/handling & applicable taxes).........See the ORDER.FRM file.........", .13 )
  oText:display()
  DO WHILE (nKey := INKEY()) == 0
     oText:displayMarquee()
  ENDDO
  oText:hide(8)
  oText:kill()
RETURN( NIL )

**
*  Demo support functions
**
FUNCTION openDBFs
// ensure all DFBs needed during the demo are open
  FIELD last_name
  USE people NEW
  INDEX ON UPPER(last_name) TO people

  USE orders NEW
  INDEX ON UPPER(last_name) TO orders

  USE history NEW
  INDEX ON UPPER(last_name) TO history

RETURN( NIL )


STATIC FUNCTION CreateTB( nWhich )
// I got tired of duplicating the same TBWin object all over the place....
  LOCAL oTB
  DO CASE

    CASE nWhich = 1
         SELECT people
         oTB := TBWin():new(  3, 4, 11, 30, "w/b")
         oTB:editColor := "w+/gr"
         oTB:title := " Customers "
         oTB:addField("last_name", "@!")

    CASE nWhich = 2
         SELECT orders
         oTB := TBWin():new(  5, 34, 13, 74, "n/bg")
         oTB:editColor := "bg+/g"
         oTB:title := " Orders "
         oTB:addField("last_name", "@!")  ;   oTB:addField("quantity")
         oTB:addField("item")             ;   oTB:addField("price")

    CASE nWhich = 3
         SELECT history
         oTB := TBWin():new(  15, 34, 23, 74, "w+/g")
         oTB:editColor := "w+/rb"
         oTB:title := " History "
         oTB:addField("last_name", "@!")  ;   oTB:addField("quantity")
         oTB:addField("item")             ;   oTB:addField("price")

  ENDCASE
RETURN( oTB )

STATIC FUNCTION CreateRead( nWhich )
// for the demo only: create some read objects on demand
  LOCAL oRead
  FIELD last_name, address, city, postal, telephone
  FIELD item, quantity, price
  DO CASE
    CASE nWhich = 1
         SELECT people
         oRead   := Window():new(12, 10, 20, 50, "w+/r")
         oRead:title := " Edit Customers "
         @0,1 SAY "Last Name: " GET last_name PICTURE "@!" IN oRead
         @2,1 SAY "Address:   " GET address   IN oRead
         @3,1 SAY "City:      " GET city      IN oRead
         @4,1 SAY "Postal:    " GET postal    IN oRead
         @6,1 SAY "Telephone: " GET telephone PICTURE "@R 999-999-9999" IN oRead
    CASE nWhich = 2
         SELECT orders
         oRead := Window():new(  12, 30, 16, 50, "w/g+")
         oRead:title := " Edit Orders "
         @0,1 SAY "Item:     "  GET item     IN oRead
         @1,1 SAY "Quantity: "  GET quantity IN oRead
         @2,1 SAY "Price:    "  GET price    IN oRead
  ENDCASE
RETURN( oRead )


STATIC FUNCTION StuffKeys( aKeysIn )
// stuffs the keyboard with simulated keystrokes for the demo autoMode.
  STATIC aKeys := {}
  IF aKeysIn <> NIL
     CLEAR TYPEAHEAD
     aKeys := aKeysIn
  ENDIF
  IF LEN(aKeys) > 0
     // This uses the Nanforum Toolkit function to stuff the keyboard
     // because the KEYBOARD command will not stuff function keys
     FT_PutKey(aKeys[1])
     ADEL(aKeys,1) ; ASIZE( aKeys, LEN(aKeys)-1 )
  ELSE
     KEYBOARD CHR(K_ESC)
  ENDIF
RETURN( NIL )


STATIC FUNCTION ContMsg
// This returns the approprriate continuation string shown in all the
// windows during the demo.
RETURN( IIF( autoMode, "Press Alt-C to abort demo...", "Press any key to continue..." ) )

STATIC FUNCTION ConfirmQuit
  LOCAL cStartColor := SETCOLOR("W+/R")
  LOCAL altcsave := SETKEY( K_ALT_C, NIL )
  IF YNConf("Are you sure you want to abort the demo?")
     // kill all objects
     Timer():killAll() ; Window():killAll() ; Idle():killAll()
     DeskTop()
     dmoGoodbye()
     SET COLOR TO
     SET CURSOR ON
     CLEAR
     QUIT
  ENDIF
  SETCOLOR(cStartColor)
  SETKEY( K_ALT_C, altcsave )
RETURN( NIL )

STATIC FUNCTION ft_putkey
RETURN(NIL)


FUNCTION whichget( GetList )
  LOCAL nWhich := ASCAN( getList, {|g| g:hasFocus } )
  messagebox("Current get is "+transform(nWhich,"999"))
RETURN( .T. )


