* Author: Mark Vodhanel, CIS - 72456,463
* Created: 2/13/1989 at 20:25
*
* Description: System error handler for multi-user systems.
*
*			Usage:
*					ON ERROR DO syserr WITH error(), message(), message(1)
*
*										DBF file, for logging of unexpected errors, and
*										unrecoverable errors:
*																SYSERLOG.DBF
*
*										Passed as parameters:
*																error()
*																message()
*																message(1)
*
*										Passed in public variable:
*																m0noabort	(should be set before writes)
*																		.t. indicates an abort is not allowed.
*																		.f. indicates an abort is allowed.
*
*										Returned via public variable:
*																m0abort	(should be set to .f. before writes)
*																		.t. indicates an aborted lock attempt.
*																		.f. indicates a succesfull lock.
*
*
*										System variable 'DEBUG'
*																if a system variable named DEBUG is found
*																(it may be set to any value) the error
*																will be displayed, otherwise the user is
*																just informed of its sequence number and
*																then RETURN TO MASTER.
*
*
*	This error handler is designed to be used with some sort of a menu as the
* top program.  This menu should call a program or procedure and upon return
* repaint the screen and reset the ON ERROR back to this program.
*
*
*-----------------------------------------------------------------------------
*
PARAMETERS errornum, errormsg, errorsrc
PRIVATE ;
	curarea, curdbf, i, ii, mseqno, mdescrip, mdetail, pause, scrnbuff, mans ;
	message, debug

RELEASE m0abort			&& public variable indicating an aborted lock attempt,
PUBLIC m0abort			&& all write attempts that need to know if the action
m0abort = .f.				&& was successful should monitor this variable.  Must
										&& be set to .f. before all writes.

* system directory
m0sysdr = ''

* macros, might want to make these publics instead
m0reverse = 'SET COLOR TO N/W'
m0normal  = 'SET COLOR TO W/N'
m0intense = 'SET COLOR TO +W/N'
m0error   = 'SET COLOR TO +W/N'
m0curson =	'?? sys(2002,1)'
m0cursoff = '?? sys(2002)'

*** main case structure
DO CASE

		CASE errornum = 1705 .or. errornum = 108		&& file in use
			 m0abort = .f.
			 IF len(errorsrc) <= 80
				 message = errorsrc
			 ELSE
				 message = substr(errorsrc,1,80)
			 ENDIF
			 SAVE SCREEN TO scrnbuff
			 &m0reverse
			 @ 0,0
 			 @ 0,40 - (len(message)/2) SAY message
			 IF m0noabort
					* no abort allowed
					@ 24,0
					@ 24,25  SAY 'File is in use, retrying......'
				  pause = inkey(10)	&& retry rate
					RESTORE SCREEN FROM scrnbuff
					RETRY
			 ELSE
				  * allow abort
					@ 24,0
					@ 24,16  SAY "File is in use, retrying...... press 'A' to Abort"
				  pause = inkey(10)	&& retry rate
					RESTORE SCREEN FROM scrnbuff
					IF pause = 97 .or. pause = 65
						m0abort = .t.
						RETURN
					ENDIF
					RETRY
			 ENDIF

		CASE errornum = 109		&& record in use
			 m0abort = .f.
			 message = 'File: ' + dbf() + ', Record: ' + ltrim(str(recno()))
			 IF len(message) > 80
				 message = substr(message,1,80)
			 ENDIF
			 SAVE SCREEN TO scrnbuff
			 &m0reverse
			 @ 0,0
			 @ 0,40 - (len(message)/2) SAY message
			 IF m0noabort
					* no abort allowed
					@ 24,0
					@ 24,24  SAY 'Record is in use, retrying......'
				  pause = inkey(5)	&& retry rate
					RESTORE SCREEN FROM scrnbuff
					RETRY
			 ELSE
				  * allow abort
					@ 24,0
					@ 24,15  SAY "Record is in use, retrying...... press 'A' to Abort"
				  pause = inkey(5)	&& retry rate
					RESTORE SCREEN FROM scrnbuff
					IF pause = 97 .or. pause = 65
						m0abort = .t.
						RETURN
					ENDIF
					RETRY
			 ENDIF

		OTHERWISE
			* unexpected, or unrecoverable error handler:
			* since I don't know what caused this error, or, it is an error that
			* I am not going to try to recover from, I am going to collect all
			* the information about the current work area DBF and NDX files so
			* that I may subsequently use this area to open the SYSERLOG to record
			* this information as well as the rest of the pertinant information.

			* index array for current indices
			RELEASE curndx
			DIMENSION curndx(7)

			* inform user
			&m0reverse
			@ 24,0
			@ 24,9  SAY 'An unexpected error has occured, saving info to Error Log....'
			?? chr(7)
			pause = inkey(.5)
			?? chr(7)
			pause = inkey(.5)
			?? chr(7)
			pause = inkey(5)

			* save current area and DBF
			curarea = ltrim(str(select()))
			curdbf = dbf()

			* determine reporting mode
			IF len(trim(getenv('DEBUG'))) > 0
				ON ERROR  ? message()
				mans = 'N'
				&m0curson
				&m0reverse
				@ 24,0
				@ 24,0 SAY 'Suspend before viewing error report? ' GET mans PICTURE 'Y'
				READ
				IF mans	= 'Y'
					&m0normal
					CLEAR
					SUSPEND
				ENDIF
				debug = .t.
			ELSE
				debug = .f.
			ENDIF

			* save current indices
			i =	1
			DO WHILE i <= 7
				curndx(i) = ndx(i)
				i = i + 1
			ENDDO

			* open the error log, I need to handle to slim possibility of
			* concurrent error logging, so I am opening the error log exclusively
			* so that the errors will be processed in sequential batches.
			* This is necessary to keep the sequence numbers from getting
			* corrupted in the event of 3 or more concurrent errors.

			ON ERROR pause = inkey(5)								&& setup retry rate (5 secs)
			USE &m0sysdr.SYSERLOG ALIAS errlogf
			DO WHILE ! flock()								 			&& retry until locked
				USE &m0sysdr.SYSERLOG ALIAS errlogf
			ENDDO
			IF ! debug
				ON ERROR RETURN TO MASTER							&& in case of an error in
			ELSE																		&& this error handler
				ON ERROR
			ENDIF

			IF eof()
				* no records so make sequence number 1
				mseqno = 1
			ELSE
				* get next sequence number
				GOTO BOTTOM
				mseqno = seqno + 1
			ENDIF

			* give user the sequence number
			&m0reverse
			@ 24,0
			@ 24,7 SAY 'Sequence # ' + ltrim(str(mseqno)) + ;
				'.  Please use this number to reference this error.'

			* date and time stamp
			mdescrip = '  Date and Time'
			mdetail = dtoc(date()) + ', ' + time()
			APPEND BLANK
			REPLACE NEXT 1 seqno WITH mseqno, descrip WITH mdescrip, detail WITH mdetail

			* system error number
			mdescrip = '  Sys error number'
			mdetail = ltrim(str(errornum))
			APPEND BLANK
			REPLACE NEXT 1 seqno WITH mseqno, descrip WITH mdescrip, detail WITH mdetail

			* system error message
			mdescrip = '  Sys error message'
			mdetail = errormsg
			APPEND BLANK
			REPLACE NEXT 1 seqno WITH mseqno, descrip WITH mdescrip, detail WITH mdetail

			* system error source
			mdescrip = '  Sys error source'
			mdetail = errorsrc
			APPEND BLANK
			REPLACE NEXT 1 seqno WITH mseqno, descrip WITH mdescrip, detail WITH mdetail

			* header
			APPEND BLANK
			REPLACE NEXT 1 seqno WITH mseqno, descrip WITH '', detail WITH ''
			mdescrip = 'PROCEDURE REPORT'
			mdetail = '-------------------------------------------------------'
			APPEND BLANK
			REPLACE NEXT 1 seqno WITH mseqno, descrip WITH mdescrip, detail WITH mdetail

			* program nesting tree
			indent = ''
			i = 1
			DO WHILE len(sys(16,i+1)) <> 0
				mdescrip = '  Program level ' + ltrim(str(i))
				mdetail = indent + sys(16,i)
				APPEND BLANK
				REPLACE NEXT 1 seqno WITH mseqno, descrip WITH mdescrip, detail WITH mdetail
				i = i + 1
				indent = indent + space(2)
			ENDDO

			* header
			APPEND BLANK
			REPLACE NEXT 1 seqno WITH mseqno, descrip WITH '', detail WITH ''
			mdescrip = 'DATABASE REPORT'
			mdetail = '-------------------------------------------------------'
			APPEND BLANK
			REPLACE NEXT 1 seqno WITH mseqno, descrip WITH mdescrip, detail WITH mdetail

			* current work area
			mdescrip = '  Current work area'
			mdetail = curarea
			APPEND BLANK
			REPLACE NEXT 1 seqno WITH mseqno, descrip WITH mdescrip, detail WITH mdetail

			* cycle through all the work areas and possible indices
			i = 1
			i_str = '1'
			DO WHILE i <= 10
				* DBF's
				IF len(dbf(i)) <> 0
					IF val(curarea) = i
						* this is the previously saved current area, so use the info
						* stored in variables: curdbf and curndx(i)
						mdescrip = '  DBF for area ' + ltrim(str(i))
						mdetail = curdbf
					ELSE
						mdescrip = '  DBF for area ' + ltrim(str(i))
						mdetail = dbf(i)
					ENDIF
					SELECT errlogf
				 	APPEND BLANK
					REPLACE NEXT 1 seqno WITH mseqno, descrip WITH mdescrip, detail WITH mdetail
					ii = 1
					* NDX's
					SELECT &i_str
					DO WHILE len(trim(ndx(ii))) <> 0 .or. ;
						(val(curarea) = i .and. len(trim(curndx(ii))) <> 0)
						IF val(curarea) = i
							* this is the previously saved current area, so use the info
							* stored in variables: curdbf and curndx(i)
							mdescrip = '   Index ' + ltrim(str(ii))
							mdetail = space(4) + curndx(ii)
							SELECT errlogf
							APPEND BLANK
							REPLACE NEXT 1 seqno WITH mseqno, descrip WITH mdescrip, detail WITH mdetail
						ELSE
							mdescrip = '   Index ' + ltrim(str(ii))
							mdetail = space(4) + ndx(ii)
							SELECT errlogf
							APPEND BLANK
							REPLACE NEXT 1 seqno WITH mseqno, descrip WITH mdescrip, detail WITH mdetail
						ENDIF
						SELECT &i_str
						ii = ii + 1
					ENDDO
				ENDIF
				i = i + 1
				i_str = ltrim(str(i))
			ENDDO
			SELECT errlogf

			* header
			APPEND BLANK
			REPLACE NEXT 1 seqno WITH mseqno, descrip WITH '', detail WITH ''
			mdescrip = 'SYSTEM INFO'
			mdetail = '-------------------------------------------------------'
			APPEND BLANK
			REPLACE NEXT 1 seqno WITH mseqno, descrip WITH mdescrip, detail WITH mdetail

	 		* memory report
			RELEASE ;
				curarea, curdbf, i, ii, curndx
			mdescrip = '  Remaining memory'
			mdetail = sys(12)
			APPEND BLANK
			REPLACE NEXT 1 seqno WITH mseqno, descrip WITH mdescrip, detail WITH mdetail

			* processor
			mdescrip = '  Processor'
			mdetail = sys(17)
			APPEND BLANK
			REPLACE NEXT 1 seqno WITH mseqno, descrip WITH mdescrip, detail WITH mdetail

			* operating system
			mdescrip = '  Operating System'
			mdetail = os()
			APPEND BLANK
			REPLACE NEXT 1 seqno WITH mseqno, descrip WITH mdescrip, detail WITH mdetail

			***********************************************************************
			* record public variables
			***********************************************************************

			* header
			APPEND BLANK
			REPLACE NEXT 1 seqno WITH mseqno, descrip WITH '', detail WITH ''
			mdescrip = 'PUBLIC VARIABLES'
			mdetail = '-------------------------------------------------------'
			APPEND BLANK
			REPLACE NEXT 1 seqno WITH mseqno, descrip WITH mdescrip, detail WITH mdetail

			* install publics that you want to record using this skeleton
			* public variable
			* mdescrip = '  var_name'
			* mdetail = varname	&& make sure this is a string
			* APPEND BLANK
			* REPLACE NEXT 1 seqno WITH mseqno, descrip WITH mdescrip, detail WITH mdetail

			* header
			mdescrip = 'END SEQ #' + ltrim(str(mseqno)) 
			mdetail = '-------------------------------------------------------'
			APPEND BLANK
			REPLACE NEXT 1 seqno WITH mseqno, descrip WITH mdescrip, detail WITH mdetail

			UNLOCK	&& no more exclusive use needed

			* display error if environment variable DEBUG exists
			IF debug
				 &m0normal
				 SET MENU OFF
				 GOTO TOP
				 tempf = m0sysdr + sys(3)
				 COPY TO &tempf FOR seqno = mseqno
				 USE &tempf ALIAS ERROR_LOG
				 BROWSE FIELDS descrip, detail NOMODIFY
				 DELETE FILE &tempf
			ELSE
				pause = inkey(0)	&& pause to let user jot down sequence number
			ENDIF

		  * abort and return to top
			m0abort = .t.
		 	CLOSE DATABASES
			FLUSH
			RETURN TO MASTER

ENDCASE

